%option noyywrap
%option noinput
%option nounput
%option reentrant
%option extra-type="struct px_url*"

%{
#ifdef __linux__
#include <stdlib.h>
#else
#include <sys/malloc.h>
#endif /* __linux__ */
#include <ctype.h>
#include <string.h>
#include <px_url.h>

%}

/* TODO this state-use scheme seems fragile, put some more effort into
 * evaluating its correctness
 */
%x SCHEME
%x USERINFO
%x HOST
%x PORT
%x PATH
%x QUERY
%x FRAGMENT

%% // start of rules
      
  /* scheme is only at the beginning of a line */
<INITIAL>^[[:space:]]*[a-z]{1,}:\/\/ { 
  BEGIN(SCHEME);
  char const* c = yytext;
  while (isspace((unsigned char)*c))
    ++c;
  if (yyextra->scheme)
    free(yyextra->scheme);
  yyextra->scheme = strdup(c);
}

<INITIAL,SCHEME>[a-z0-9]{1,}(:[a-z0-9]*)?@ {
  BEGIN(USERINFO);
  if (yyextra->userinfo)
    free(yyextra->userinfo);
  yytext[yyleng-1] = '\0';
  yyextra->userinfo = strdup(yytext);
}

<INITIAL,SCHEME,USERINFO>[a-zA-Z0-9.-]{2,} {
  BEGIN(HOST);
  if (yyextra->host)
    free(yyextra->host);
  yyextra->host = strdup(yytext); 
}

<HOST>:[0-9]{1,5} {
  BEGIN(PORT);
  if (yyextra->port)
    free(yyextra->port);
  yyextra->port = strdup(&yytext[1]); 
}

<INITIAL,SCHEME,HOST,PORT>\/[^#?]* {
  BEGIN(PATH);
  if (yyextra->path)
    free(yyextra->path);
  yyextra->path = strdup(yytext); 
}

<INITIAL,SCHEME,HOST,PORT,PATH>\? {
  BEGIN(QUERY);
}

<QUERY>[a-zA-Z0-9_=&%-]* {
  if (yyextra->query)
    free(yyextra->query);
  yyextra->query = strdup(yytext);
}

<INITIAL,SCHEME,PATH,QUERY># {
  BEGIN(FRAGMENT);
}

<FRAGMENT>[a-zA-Z0-9_%]+ {
  if (yyextra->fragment)
    free(yyextra->fragment);
  yyextra->fragment = strdup(yytext);
}

<INITIAL,SCHEME,USERINFO,HOST,PORT,PATH,QUERY,FRAGMENT>. { fprintf(stderr, "Unrecognized item: %s\n", yytext); return -1; }
  
%% // user code

int lexurl_r(struct px_url *url, char const *buf, int buflen) {
  int ret = 0;
	if (url == NULL || buflen < 0)
		return -1;

  yyscan_t scanner;
  yylex_init_extra(url, &scanner);
	YY_BUFFER_STATE st = yy_scan_bytes(buf, buflen, scanner);
	if (yylex(scanner) != 0)
    ret = -1;
  yyurl_delete_buffer(st, scanner);
  yyurllex_destroy(scanner);

	return ret;	
}
