#!/bin/sh

OUTDIR=${1:-/tmp}

die() {
  echo "$@"
  exit -1
}

git status 2>/dev/null >&2 || die "This script needs 'git archive' to work"

[ -d "$OUTDIR" ] || die "output directory $OUTDIR is not a directory"

make 2>/dev/null >&2 || die "could not build the project"

[ -f "version.txt" ] || die "no version.txt, it needs to be included in archives"

TAG=""
if [ "`git status -uno --porcelain 2>/dev/null | wc -l 2>/dev/null`" -ne 0 ] ; then
  TAG="`git rev-list --abbrev-commit --max-count=1 HEAD 2>/dev/null`"
  TAG="${TAG}-devel"
else
  TAG="`git tag --contains HEAD 2>/dev/null`"
  if [ -z "$TAG" ] ; then
    TAG="`git rev-list --abbrev-commit --max-count=1 HEAD 2>/dev/null`"
  fi
fi

PREFIX=pxc-${TAG:-git}
OUTFILE="${OUTDIR}/${PREFIX}.tar.gz"

git archive --prefix="${PREFIX}/" --add-file version.txt -o "$OUTFILE" HEAD || die "Could not create archive"
echo "$OUTFILE"
