//#include <px_common.h>
#include <px_url.h>
#include <stdlib.h>
#include <string.h>
//#include <px_log.h>
#include <errno.h>
#include <ctype.h>
#include <stdio.h>
#include <assert.h>

struct px_url* px_url_new() {
  return (struct px_url*)calloc(1, sizeof(struct px_url));
}

void px_url_init(struct px_url* url) {
  memset(url, 0, sizeof(*url));
}

void px_url_reset(struct px_url* url) {
  if (!url)
    return;
  free(url->scheme);
  free(url->userinfo);
  free(url->host);
  free(url->port);
  free(url->path);
  free(url->query);
  free(url->fragment);
  px_url_init(url);
}

int lexurl_r(struct px_url *url, char const *buf, int buflen);

int px_url_parse(struct px_url* url, char const* buf, unsigned buflen) {
  int r;

  if (!buf || buflen == 0)
    return 0;

  r = lexurl_r(url, buf, buflen);
  if (r != 0) {
    fprintf(stderr, "could not parse url from: %.*s\n", buflen, buf);
    return -1;
  }
  return 0;
}

// 1024 bytes max, up to 4 printed chars per url char, \r\n\0
#define MAX_URL_STR_SIZE (1024 * 4 + 3)
static char urlstrbuf[MAX_URL_STR_SIZE];
char const* px_url_to_str(struct px_url const* url) {
  char const* nullstr = "(null)";
  if (!url)
    return nullstr;

  memset(&urlstrbuf[0], 0, sizeof(urlstrbuf));
  char* bufpos = &urlstrbuf[0];
  char* bufend = &urlstrbuf[MAX_URL_STR_SIZE - 2];

  char const* fields[] = { //url->scheme,
                           //url->userinfo,
                           //url->host,
                           //url->port,
                           url->path };
                           //url->query,
                           //url->fragment };
  const int n_fields = sizeof(fields) / sizeof(fields[0]);

  for (char const** cur_field = &fields[0], **end_field = &fields[n_fields];
       cur_field != end_field;
       ++cur_field)
  {
    char const* fieldstr = *cur_field;
    if (fieldstr == NULL)
      continue;

    for (unsigned char c = *fieldstr; c != '\0'; ++fieldstr, c = *fieldstr) {
      if (isgraph(c)) {
        *bufpos++ = c;
      } else {
        size_t sz = bufend - bufpos;
        assert(sz > 0);
        int r = snprintf(bufpos, sz, "(%02hhx)", c);
        assert(r <= sz);
        bufpos += r;
        assert(bufpos <= bufend);
      }
    }
  }

  return &urlstrbuf[0];
}
